//
//  TWI_I2CMaster.c
//
//  ATmega TWI I2C(master) control subroutine
//  Copyright(C)2010 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		Master mode I2C interface.
//		AVR ATmega TWI(Two-wire serial interface) internal module.
//		Interrupt nonuse, simple logic.
//		Need pull-up register.
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2010.12.20 v0.1  Write only version
//

#include <avr/io.h>						// WinAVR library
#include "TWI_I2CMaster.h"
#include <util/twi.h>					// WinAVR library 


void TWI_I2CMasterStart(void) {
  /* initialize TWI clock: 100 kHz clock, TWPS = 0 => prescaler = 1 */
#if defined(TWPS0)
  /* has prescaler (mega128 & newer) */
  TWSR = 0;
#endif

#if F_CPU < 3600000UL
  TWBR = 10;			/* smallest TWBR value, see note [5] */
#else
  TWBR = (F_CPU / 100000UL - 16) / 2;
#endif
}

#define RETRY 3

unsigned char TWI_I2CWrite(unsigned char SlaveAddress, void* TxData, unsigned int Length, unsigned char Multi) {
  	char *p;
	unsigned char rc;
	unsigned char i;
	unsigned char twst;
  
	rc = 1;

	for(i = 0; i < RETRY; i++) {

		p = TxData;

		// Send start condition
		TWCR = _BV(TWINT) | _BV(TWSTA) | _BV(TWEN);	/* send start condition */
		while ((TWCR & _BV(TWINT)) == 0) ;			/* wait for transmission */

		twst = TW_STATUS;
		if (twst == TW_MT_ARB_LOST) {
			continue;
		}
		if ((twst != TW_REP_START) && (twst != TW_START)) {
			break;
		}

		// Send slave address
		TWDR = (SlaveAddress << 1) | TW_WRITE;
		TWCR = _BV(TWINT) | _BV(TWEN);		/* clear interrupt to start transmission */

		while ((TWCR & _BV(TWINT)) == 0);	/* wait for transmission */
		twst = TW_STATUS;
		if (twst == TW_MT_SLA_NACK) {		/* nack during select: device busy writing */
			continue;
		}
		if (twst == TW_MT_ARB_LOST) {		/* re-arbitrate */
			continue;
		}
		if (twst != TW_MT_SLA_ACK) {
			break;
		}

		// send datas
		for(; Length > 0; --Length){
			TWDR = *p++;
			TWCR = _BV(TWINT) | _BV(TWEN);		/* start transmission */

			while ((TWCR & _BV(TWINT)) == 0);	/* wait for transmission */
			twst = TW_STATUS;
			if (twst != TW_MR_DATA_NACK) {
				continue;
			}
			if (twst != TW_MT_DATA_ACK) {
				break;
			}
    	}
		rc = 0;
	}

	// Send stop condition
	if (!Multi) {
		TWCR = _BV(TWINT) | _BV(TWSTO) | _BV(TWEN); /* send stop condition */
	}

	return(rc);

}

void TWI_I2CStop(void){
}
