//
//  lcd.c
//
//  Character LCD module Control
//  Copyright (C) 2010 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Character LCD module.
//		Interface: 8/4bit pararell or I2C.
//		Controller: HD44780, ST7032i and compatible.
//
//  Device: CY8C29466/27443  IDE: PSoC Designer 5 Service Pack 6(Build 1127)
//  Device: MSP430F2013      IDE: IAR Embedded Workbench for MSP430 V4.11B
//  Device: PIC18F14K50      IDE: MPLAB IDE 8.36, C Compiler for PIC18 MCUs v3.33
//  Device: ATmega328P       IDE: AVR Studio 4.18.700, WinAVR 20100110(GCC 4.3.3)
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2009.06.28 v0.0  First cording
//    2009.07.02 v0.0  Rewrite & 1st release
//    2009.07.05 v0.0  Compile error Bug fix
//    2009.07.06 v0.0  Startup wait 40ms->100ms
//    2009.08.02 v0.1  HI-TECH C -> ImageCraft C Compiler
//    2009.08.13 v0.1  Independ I2Cm and CPU Clock
//    2009.09.01 v0.2  Support "IAR EW MSP430" (Need TexasInstruments SLAA368368A Library)
//    2009.09.13 v0.3  Support "MPLAB+C18" and code reduction
//    2009.12.12 v0.3  Variable type BYTE,CHAR,INT -> uchar,char,int
//    2009.12.22 v0.3  External delay
//    2009.12.22 v0.3  Change API Name, LCD->lcd
//    2009.12.22 v0.3  Reset update history
//    2010.12.20 v0.4  Support "AVR Studio ATmega TWI" (Need TWI_I2CMaster.[ch])
//    2010.12.20 v0.4  i2c_lcd.c were integrated
//


#include "lcd.h"						// Private
#include "delay.h"						// Private

//
// Initial command & save registers
//
static unsigned char lcd_reg_func = 
            LCD_INST_FUNC
//     |    LCD_DL_8BIT
//     |    LCD_DL_4BIT
       |    LCD_N_2LINE
//     |    LCD_N_1LINE
//     |    LCD_DH_DOUBLE
       |    LCD_DH_NORMAL
//     |    LCD_IS_EXTEND
//     |    LCD_IS_NORMAL
       ;
static unsigned char lcd_reg_bias = 
            LCD_INST_BIAS
//     |    LCD_BS_4
       |    LCD_BS_5
//     |    LCD_OSC_122HZ
//     |    LCD_OSC_131HZ 
//     |    LCD_OSC_144HZ
//     |    LCD_OSC_161HZ
       |    LCD_OSC_183HZ
//     |    LCD_OSC_221HZ
//     |    LCD_OSC_274HZ
//     |    LCD_OSC_347HZ 
//     |    LCD_OSC_120HZ
//     |    LCD_OSC_133HZ
//     |    LCD_OSC_149HZ
//     |    LCD_OSC_167HZ
//     |    LCD_OSC_192HZ
//     |    LCD_OSC_227HZ
//     |    LCD_OSC_277HZ
       ;
static unsigned char lcd_reg_cont = 
          LCD_INST_CONT
     |    (LCD_C_LOW & LCD_INIT_CONT)
     ;
static unsigned char lcd_reg_pow = 
            LCD_INST_POW
       |    LCD_I_ON                // icon
//     |    LCD_I_OFF 
       |    LCD_B_ON                // booster
//     |    LCD_B_OFF 
       |    (LCD_C_HIGH & (LCD_INIT_CONT >> 4))
       ;
static unsigned char lcd_reg_foll = 
            LCD_INST_FOLL
       |    LCD_F_ON                // switch follwer circuit
//     |    LCD_F_OFF 
//     |    LCD_RAB_1               // x1.0
//     |    LCD_RAB_1_25            // x1.25
//     |    LCD_RAB_1_5             // x1.5
       |    LCD_RAB_1_8             // x1.8
//     |    LCD_RAB_2               // x2.0
//     |    LCD_RAB_2_5             // x2.5
//     |    LCD_RAB_3               // x3.0
//     |    LCD_RAB_3_75            // x3.75
    ;
static unsigned char lcd_reg_disp = 
            LCD_INST_DISP
       |    LCD_DSP_ON
       |    LCD_DSP_OFF
//     |    LCD_CSL_ON
       |    LCD_CSL_OFF
//     |    LCD_BLK_ON
//     |    LCD_BLK_OFF
       ;

//
// Internal function
//
char lcd_itoc(int i) {
	return((i <= 9) ? (i+'0') : (i+'a'-10));
}

//
// API
//
void lcd_start(void) {
	//
	// Parallel I/F
	//
#ifdef LCD_RS_PIN
	#if LCD_4BIT == 0
	LCD_TRIS &= 0xf0;	// Lower 4bit output
	#else
	LCD_TRIS &= 0x0f;	// Upper 4bit output
	#endif
	LCD_E_PIN   = 0;
	LCD_E_TRIS  = 0;
	LCD_RS_PIN  = 0;
	LCD_RS_TRIS = 0;
	#ifdef LCD_RW_PIN
	LCD_RW_PIN  = 0;
	LCD_RW_TRIS = 0;
	#endif
#else
	//
	// I2C
	//
	LCD_START();
#endif
}

void lcd_stop(void) {
	//
	// Parallel I/F
	//
#ifdef LCD_RS_PIN
	#if LCD_4BIT == 0
	LCD_TRIS |= 0x0f;	// Lower 4bit Hi-Z
	#else
	LCD_TRIS |= 0xf0;	// Upper 4bit Hi-Z
	#endif
	LCD_E_TRIS  = 1;
	LCD_RS_TRIS = 1;
	#ifdef LCD_RW_PIN
	LCD_RW_TRIS = 1;
	#endif
#else
	//
	// I2C
	//
	LCD_STOP();
#endif
}

void lcd_sendCmdData(unsigned char ctl, unsigned char c) {
	//
	// Parallel I/F
	//
#ifdef LCD_RS_PIN
	#ifdef LCD_RW_PIN
	LCD_RW_PIN = 0;		// Write
	#endif

	if (ctl == LCD_CB_INST) {
		LCD_RS_PIN = 0;		// Command
	} else {
		LCD_RS_PIN = 1;		// Data
	}

	#ifdef LCD_8BIT
	LCD_PORT = c;
	LCD_E_PIN = 1;		// Enable pulse
	delay_50u();
	LCD_E_PIN = 0;

	#elif LCD_4BIT == 0
	LCD_PORT = (LCD_PORT & 0xf0) | (c >> 4);	// High
	LCD_E_PIN = 1;		// Enable pulse
	delay_50u();
	LCD_E_PIN = 0;

	LCD_PORT = (LCD_PORT & 0xf0) | (c & 0xf);	// Low
	LCD_E_PIN = 1;		// Enable pulse
	delay_50u();
	LCD_E_PIN = 0;

	#else
	LCD_PORT = (LCD_PORT & 0x0f) | (c & 0xf0);	// High
	LCD_E_PIN = 1;		// Enable pulse
	delay_50u();
	LCD_E_PIN = 0;

	LCD_PORT = (LCD_PORT & 0x0f) | (c << 4);	// Low
	LCD_E_PIN = 1;		// Enable pulse
	delay_50u();
	LCD_E_PIN = 0;

	#endif

#else
	//
	// I2C
	//
	unsigned char buf[2];

	buf[0] = ctl;			// LCD_CB_xxx
	buf[1] = c;			// Data/Command byte
	LCD_WRITE(buf, sizeof(buf));
	delay_50u();                 // 26.3us~
#endif
	return;
}


void lcd_control(unsigned char cmd) {
	lcd_sendCmdData(LCD_CB_INST, cmd);
}

void lcd_writeData(unsigned char data) {
	lcd_sendCmdData(LCD_CB_DATA, data);
}


void lcd_init(void) {
#ifdef LCD_4BIT
	lcd_reg_func |= LCD_DL_4BIT;
#else
	lcd_reg_func |= LCD_DL_8BIT;
#endif
    delay_1mTimes(100);
    lcd_control(lcd_reg_func | LCD_IS_EXTEND);
    lcd_control(lcd_reg_func | LCD_IS_EXTEND);
    lcd_control(lcd_reg_bias);
    lcd_control(lcd_reg_cont);
    lcd_control(lcd_reg_pow);
    lcd_control(lcd_reg_foll);
    delay_1mTimes(200);          // 200ms
    lcd_control(lcd_reg_disp);
    lcd_control(lcd_reg_func | LCD_IS_NORMAL);

    return;
}

// Override stdio -> usage: putchar(),puts(),printf()
//void  putch(char c){
//    lcd_writeData(c);
//    return;
//}

void lcd_prString(char *s) {
    for(; *s != '\0'; s++) {
        lcd_writeData(*s);
    }
    return;
}

void lcd_prCString(const char *s) {
    for(; *s != '\0'; s++) {
        lcd_writeData(*s);
    }
    return;
}

void lcd_prHexByte(unsigned char b) {
    char buf[1];

    buf[0] = lcd_itoc(b >> 4);
    lcd_writeData(buf[0]);
    buf[0] = lcd_itoc(b & 0xf);
    lcd_writeData(buf[0]);
    return;
}

void lcd_prHexInt(int i) {
    lcd_prHexByte(i >> 8);
    lcd_prHexByte(i & 0xff);
    return;
}

void  lcd_position(unsigned char bRow, unsigned char bCol){
    lcd_control(LCD_INST_DADR | (bRow << 6) | (bCol));
    return;
}

void  lcd_contrast(unsigned char bCont){
    bCont &= 0x3f;
    lcd_reg_cont = (lcd_reg_cont & ~LCD_C_LOW ) | (LCD_C_LOW & bCont);
    lcd_reg_pow  = (lcd_reg_pow  & ~LCD_C_HIGH) | (bCont >> 4);
    lcd_control(lcd_reg_func | LCD_IS_EXTEND);
    lcd_control(lcd_reg_cont);
    lcd_control(lcd_reg_pow);
    lcd_control(lcd_reg_func | LCD_IS_NORMAL);
    return;
}

void  lcd_icon(unsigned char bAddr, unsigned char bFlag){
    lcd_control(lcd_reg_func | LCD_IS_EXTEND);
    lcd_control(LCD_INST_ICON | (bAddr & LCD_ICON_ADDR));
    lcd_writeData(bFlag);
    lcd_control(lcd_reg_func | LCD_IS_NORMAL);
    return;
}
