//
//  lcd.h
//
//  Character LCD module Control
//  Copyright (C) 2010 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Character LCD module.
//		Interface: 8/4bit pararell or I2C.
//		Controller: HD44780, ST7032i and compatible.
//
//  Device: CY8C29466/27443  IDE: PSoC Designer 5 Service Pack 6(Build 1127)
//  Device: MSP430F2013      IDE: IAR Embedded Workbench for MSP430 V4.11B
//  Device: PIC18F14K50      IDE: MPLAB IDE 8.36, C Compiler for PIC18 MCUs v3.33
//  Device: ATmega328P       IDE: AVR Studio 4.18.700, WinAVR 20100110(GCC 4.3.3)
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2009.06.28 v0.0  First cording
//    2009.07.02 v0.0  Rewrite & 1st release
//    2009.07.05 v0.0  Compile error Bug fix
//    2009.08.02 v0.1  HI-TECH C -> ImageCraft C Compiler
//    2009.08.13 v0.1  Independ I2Cm and CPU Clock
//    2009.09.01 v0.2  Support "IAR EW MSP430" (Need TexasInstruments SLAA368368A Library)
//    2009.09.13 v0.3  Support "MPLAB+C18"
//    2009.12.12 v0.3  Variable type BYTE,CHAR,INT -> uchar,char,int
//    2009.12.22 v0.3  External delay
//    2009.12.22 v0.3  Change API Name, LCD->lcd
//    2009.12.22 v0.3  Reset update history
//    2010.12.20 v0.4  Support "AVR Studio ATmega TWI" (Need TWI_I2CMaster.[ch])
//    2010.12.20 v0.4  i2c_lcd.c were integrated


#ifndef LCD_H
#define LCD_H

//
//    Configure
//        (Choice interface of device)
//
//#define PARA_PIC16		// Microchip PIC16/HI-TECH
//#define PARA_PIC18		// Microchip PIC18/C18
//#define   _XTAL_FREQ	4000000
//#define   LCD_PORT	PORTB		// Command/Data port
//#define   LCD_TRIS	TRISB
//#define   LCD_8BIT
//#define   LCD_4BIT	1		// CPU:UPPER -> LCD:UPPER nibble
//#define   LCD_4BIT	0		// CPU:LOWER -> LCD:UPPER nibble
//#define   LCD_E_PIN	RB4		// Flow control pin
//#define   LCD_E_TRIS	TRISB4
//#define   LCD_RS_PIN	RB5
//#define   LCD_RS_TRIS	TRISB5
//#define   LCD_RW_PIN	RA5
//#define   LCD_RW_TRIS	TRISA6
//#define I2CM_M8C_SW			// Cypress PSoC1 I2Cm_1 module
//#define I2CM_M8C_HW			// Cypress PSoC1 I2CHW_1 module (need insert "M8C_EnableGInt;" & clock 400kHz)
//#define I2CM_MSP430_USI		// Texas Instruments USI internal peripheral module
//#define I2CM_PIC18			// Microchip PIC18/C18 peripheral library
#define I2CM_AVR_TWI			// Atmel ATmega TWI internal peripheral module
//#define I2CM_AVR_USI			// Atmel ATtiny USI internal peripheral module
#define   LCD_ADDR	0x3e		// LCD slave address
//-------------------------------------------------------
#ifdef I2CM_M8C_SW
#include <m8c.h>
#include <GlobalParams.h>
#include "I2Cm_1.h"
#define LCD_START()		I2Cm_1_Start()
#define LCD_STOP()		I2Cm_1_Stop()
#define LCD_WRITE(b,c)	I2Cm_1_bWriteBytes(LCD_ADDR,b,c,I2Cm_1_CompleteXfer)
#endif  //I2CM_M8C_SW
//-------------------------------------------------------
#ifdef I2CM_M8C_HW
#include <m8c.h>
#include <GlobalParams.h>
#include "I2CHW_1Common.h"
#include "I2CHW_1Mstr.h"
#define LCD_START()		{I2CHW_1_Start();I2CHW_1_EnableMstr();I2CHW_1_EnableInt();}
#define LCD_STOP()		I2CHW_1_Stop()
#define LCD_WRITE(b,c)	I2CHW_1_bWriteBytes(LCD_ADDR,b,c,I2CHW_1_CompleteXfer)
#endif  //I2C_M8C_HW
//-------------------------------------------------------
#ifdef I2CM_MSP430_USI
#include "msp430.h"
#include "USI_I2CMaster.h"	// slaa368a Using the USI I2C Code Library
#define LCD_START()		TI_USI_I2C_MasterInit(USIDIV_7+USISSEL_2+USICKPL, 0)
#define LCD_STOP()		TI_USI_I2CStop()
#define LCD_WRITE(b,c)	TI_USI_I2CWrite(LCD_ADDR, c, 0, b)
#endif  // I2CM_MSP430_USI
//-------------------------------------------------------
#ifdef I2CM_PIC18
#include <i2c.h>
#define const  const rom
#define LCD_START()		{OpenI2C(MASTER, SLEW_OFF);	SSPADD = 39;} // FOSC=16MHz SCL->100KHz
#define LCD_STOP()		CloseI2C()
#define LCD_WRITE(b,c)	EEByteWrite(LCD_ADDR<<1, b[0], b[1])
#endif  // I2CM_PIC18
//-------------------------------------------------------
#ifdef I2CM_AVR_TWI
#include "TWI_I2CMaster.h"
#define LCD_START()		TWI_I2CMasterStart()
#define LCD_STOP()		TWI_I2CStop()
#define LCD_WRITE(b,c)	TWI_I2CWrite(LCD_ADDR, b, c, 0)
#endif // I2CM_AVR_TWI
//-------------------------------------------------------


//
//    A P I
//

// standard / compatible
extern void lcd_start(void);
extern void lcd_stop(void);
extern void lcd_init(void);
extern void lcd_position(unsigned char bRow, unsigned char bCol);
extern void lcd_control(unsigned char c);
extern void lcd_writeData(unsigned char d);
extern void lcd_prString(char * sRamString);
extern void lcd_prCString(const char * sRomString);
extern void lcd_prHexByte(unsigned char bValue);
extern void lcd_prHexInt(int iValue);

// special
extern void lcd_contrast(unsigned char bCont);
extern void lcd_icon(unsigned char bAddr, unsigned char bFlag);
extern void lcd_sendCmdData(unsigned char cmd, unsigned char data);


//
// Control command
//
// PSoC Designer LCD module compatible define
#define LCD_DISP_ON                       0x0C
#define LCD_DISP_OFF                      0x08
#define LCD_DISP_BLANK                    0x0A
#define LCD_DISP_CLEAR_HOME               0x01
#define LCD_CURSOR_ON                     0x0E
#define LCD_CURSOR_OFF                    0x0C
#define LCD_CURSOR_WINK                   0x0D
#define LCD_CURSOR_BLINK                  0x0F
#define LCD_CURSOR_SH_LEFT                0x10
#define LCD_CURSOR_SH_RIGHT               0x14
#define LCD_CURSOR_HOME                   0x02
#define LCD_CURSOR_LEFT                   0x04
#define LCD_CURSOR_RIGHT                  0x06
#define LCD_PORT_MASK                     0x7F

// ST7032i I2C stream
#define LCD_CB_CONT   0x80          // I2C Control byte -> Continue stream
#define LCD_CB_INST   0x00          // I2C Control byte -> next byte is instruction
#define LCD_CB_DATA   0x40          // I2C Control byte -> next byte is data

// Instruction: "Function Set"
#define LCD_INST_FUNC 0x20          // ID
#define LCD_DL_8BIT   0x10          // DL: Interface data length
#define LCD_DL_4BIT   0x00          //
#define LCD_N_2LINE   0x08          // N: Display line number  1=2-line 0=1-line
#define LCD_N_1LINE   0x00          //
#define LCD_DH_DOUBLE 0x04          // DH: Double height font type
#define LCD_DH_NORMAL 0x00          //
#define LCD_IS_EXTEND 0x01          // IS: normal/extension instruction select
#define LCD_IS_NORMAL 0x00          //

//    Instruction: <extention mode> "Bias selection / Internal OSC frequency adjust"
#define LCD_INST_BIAS 0x10          // ID
#define LCD_BS_4      0x08          // BS: bias selection  1/4
#define LCD_BS_5      0x00          //                     1/5
#define LCD_OSC_122HZ 0x00          // Internal OSC frequency adjust
#define LCD_OSC_131HZ 0x01          // 
#define LCD_OSC_144HZ 0x02          // 
#define LCD_OSC_161HZ 0x03          // 
#define LCD_OSC_183HZ 0x04          // 
#define LCD_OSC_221HZ 0x05          // 
#define LCD_OSC_274HZ 0x06          // 
#define LCD_OSC_347HZ 0x07          // 
#define LCD_OSC_120HZ 0x00          // Vdd=5.0V
#define LCD_OSC_133HZ 0x01          // Vdd=5.0V
#define LCD_OSC_149HZ 0x02          // Vdd=5.0V
#define LCD_OSC_167HZ 0x03          // Vdd=5.0V
#define LCD_OSC_192HZ 0x04          // Vdd=5.0V
#define LCD_OSC_227HZ 0x05          // Vdd=5.0V
#define LCD_OSC_277HZ 0x06          // Vdd=5.0V

//    Instruction: <extention mode> "Set ICON address"
#define LCD_INST_ICON 0x40          // ID
#define LCD_ICON_ADDR 0x0f          // adress mask bit, next data is icon patturn bit

//    Instruction: <extention mode> "Power/ICON control/Contrast set"
#define LCD_INST_POW  0x50          // ID
#define LCD_I_ON      0x08          // Ion: set ICON display
#define LCD_I_OFF     0x00          // 
#define LCD_B_ON      0x04          // Bon: switch booster circuit
#define LCD_B_OFF     0x00          // 
#define LCD_C_HIGH    0x03          // C5~4: Contrast value high nibble

//    Instruction: <extention mode> "Follower control"
#define LCD_INST_FOLL 0x60          // ID
#define LCD_F_ON      0x08          // switch follwer circuit
#define LCD_F_OFF     0x00          // 
#define LCD_RAB       0x07          // Rab2-0: V0 generator amplified ratio
#define LCD_RAB_1     0x00          // x1.0
#define LCD_RAB_1_25  0x01          // x1.25
#define LCD_RAB_1_5   0x02          // x1.5
#define LCD_RAB_1_8   0x03          // x1.8
#define LCD_RAB_2     0x04          // x2.0
#define LCD_RAB_2_5   0x05          // x2.5
#define LCD_RAB_3     0x06          // x3.0
#define LCD_RAB_3_75  0x07          // x3.75

//    Instruction: <extention mode> "Contrast set"
#define LCD_INST_CONT 0x70          // ID
#define LCD_C_LOW     0x0f          // C3~0: Contrast value low nibble

//    Instruction: "Display ON/OFF"
#define LCD_INST_DISP 0x08          // ID
#define LCD_DSP_ON    0x04          // Diaplay ON/OFF control
#define LCD_DSP_OFF   0x00          // 
#define LCD_CSL_ON    0x02          // Cursor ON/OFF control
#define LCD_CSL_OFF   0x00          // 
#define LCD_BLK_ON    0x01          // Cursor blink ON/OFF control
#define LCD_BLK_OFF   0x00          // 

//    Instruction: "Set DDRAM address"
#define LCD_INST_DADR 0x80          // ID
#define LCD_POS_LINE1 0x00          // col=0, line=0
#define LCD_POS_LINE2 0x40          // col=0, line=1

//
// Icon symbol
//
#define LCD_ICON_ANTENNA       0x00
#define LCD_ICON_TELL          0x02
#define LCD_ICON_SIGNAL        0x04
#define LCD_ICON_ARROW         0x06
#define LCD_ICON_UPDOWN        0x07
#define LCD_ICON_LOCK          0x09
#define LCD_ICON_NG            0x0b
#define LCD_ICON_BATTERY       0x0d
#define LCD_ICON_TIMER         0x0f

#define LCD_ICON_FLAG_OFF      0x00
#define LCD_ICON_FLAG_ON       0x1f
#define LCD_ICON_FLAG_ON1      0x10
#define LCD_ICON_FLAG_ON2      0x08
#define LCD_ICON_FLAG_ON3      0x04
#define LCD_ICON_FLAG_ON4      0x02
#define LCD_ICON_FLAG_ON5      0x01
#define LCD_ICON_UPDOWN_UP     0x10
#define LCD_ICON_UPDOWN_DOWN   0x08
#define LCD_ICON_BATTERY_EMPTY 0x02
#define LCD_ICON_BATTERY_LOW   0x12
#define LCD_ICON_BATTERY_MOD   0x0a
#define LCD_ICON_BATTERY_HIGH  0x1a
#define LCD_ICON_BATTERY_FULL  0x1e

//
// Initial data
//
//#define LCD_INIT_CONT    32       // Contrast 3.6V
//#define LCD_INIT_CONT    48       //  (0~63)  3.3V
#define LCD_INIT_CONT      50       //          3.3V
//#define LCD_INIT_CONT    62       //          2.7V
//#define LCD_INIT_CONT    40       //          3.5V

#endif  // LCD_H
